import * as React from 'react';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { History } from 'history';
import { findLast, propEq } from 'ramda';

import Modals from '../../Modal/components/Modals';
import Toasters from '../../Toaster/components/Toasters';

import { IMenuItem } from '../types';
import {
	StyledLeftMenuLabel,
	StyledLeftMenu,
	StyledLeftMenuItem,
	StyledRightSide,
	StyledTopMenu,
	StyledPageContent,
	StyledLeftMenuLogo,
	StyledTopMenuItem,
	StyledTopLeftMenuContainer,
	StyledTopMenuItemHamburger,
	StyledTopMenuItemLogo,
	StyledTopMenuDisplayNextItemIcon,
	StyledTopMenuDisplayNextItem,
} from '../style';
import { getComputedStyleInt } from '../utils';


interface IProps {
	enableLeftMenu: boolean;
	history: History;
	leftMenuItems?: IMenuItem[];
	logo: React.ReactElement;
	topMenuItems?: IMenuItem[];
}

interface IState {
	maxLeftMenuItemCount: number;
	open: boolean;
}

class PageLayout extends React.PureComponent<IProps, IState> {
	public static defaultProps = {
		enableLeftMenu: true,
	};

	public readonly state = {
		open: false,
		maxLeftMenuItemCount: this.props.topMenuItems.length,
	};

	public componentDidMount() {
		this.updateDimensions();
		window.addEventListener("resize", () => {
			this.setState({
				maxLeftMenuItemCount: this.props.topMenuItems.length,
			}, this.updateDimensions);
		});
	}

	public componentDidUpdate(prevProps: IProps) {
		if (this.props.topMenuItems !== prevProps.topMenuItems) {
			this.setState({
				maxLeftMenuItemCount: this.props.topMenuItems.length,
			})
			this.updateDimensions();
		}
	}

	public componentWillUnmount() {
		window.removeEventListener("resize", this.updateDimensions);
	}

	private updateDimensions = () => {
		const fullWidth = getComputedStyleInt(this.topMenuRef.current, 'width');
		const leftSideWidth = getComputedStyleInt(this.leftSideMenuRef.current, 'width');
		const leftSideMarginRight = getComputedStyleInt(this.leftSideMenuRef.current, 'marginLeft');
		const leftMenuItems = this.getTopLeftMenuItemsWidth(this.topMenuRef.current.children);

		if (leftSideMarginRight === 0) {
			const requiredWidth = fullWidth - leftSideMarginRight - leftSideWidth;

			let newWidth = this.topMenuRef.current.scrollWidth;
			let index = 0;

			for (let i = 0; i < leftMenuItems.length; i++) {
				const itemWidth = leftMenuItems[i];
				if (newWidth - itemWidth <= requiredWidth) {
					index = i;
					break;
				}
				newWidth -= itemWidth;
			}

			this.setState({
				maxLeftMenuItemCount: leftMenuItems.length - index,
			});
		}
	}

	private getTopLeftMenuItemsWidth = (children: HTMLCollection) => {
		const sizes = [];
		for (let i = 1; i < children.length; i++) {
			const child = children[i];
			const marginSize = getComputedStyleInt(child, 'marginLeft') + getComputedStyleInt(child, 'marginRight');
			sizes.push(getComputedStyleInt(child, 'width') + marginSize);
		}

		return sizes.reverse();
	}

	private toggleOpenLeftMenu = () => {
		this.setState({
			open: !this.state.open,
		});
	};

	private closeLeftMenu = () => {
		if (this.state.open) {
			this.setState({
				open: false,
			});
		}
	};

	private openPage = (href: string) => () => {
		this.props.history.push(href);
		this.setState({
			open: false,
		});
	};

	private topMenuRef: React.RefObject<HTMLDivElement> = React.createRef();
	private leftSideMenuRef: React.RefObject<HTMLDivElement> = React.createRef();

	private renderTopNavigation = () => {
		const {
			topMenuItems,
		} = this.props;
		const {
			maxLeftMenuItemCount,
		} = this.state;

		const lastRightItem: IMenuItem = findLast(propEq('left', true))(topMenuItems);

		return (
			maxLeftMenuItemCount < topMenuItems.length
				? (
					<React.Fragment>
						<StyledTopMenuItem>
							<StyledTopMenuDisplayNextItemIcon>
								<FontAwesomeIcon icon="caret-square-down" />
								<StyledTopMenuDisplayNextItem>
									{
										topMenuItems && topMenuItems.map(
											({ label, href, left = false }, i) =>
												(!left && maxLeftMenuItemCount <= i
													|| (left && topMenuItems.length - 1 > i)) && (
													<StyledTopMenuItem key={i} onClick={this.openPage(href)}>
														{label}
													</StyledTopMenuItem>
												)
										)
									}
								</StyledTopMenuDisplayNextItem>
							</StyledTopMenuDisplayNextItemIcon>
						</StyledTopMenuItem>
						<StyledTopLeftMenuContainer ref={this.leftSideMenuRef}>
							{
								lastRightItem && (
									<StyledTopMenuItem onClick={this.openPage(lastRightItem.href)}>
										{lastRightItem.label}
									</StyledTopMenuItem>
								)
							}
						</StyledTopLeftMenuContainer>
					</React.Fragment>
				)
				: (
					<StyledTopLeftMenuContainer ref={this.leftSideMenuRef}>
						{
							topMenuItems && topMenuItems.map(
								({ label, href, left = false }, i) =>
									left && (
										<StyledTopMenuItem key={i} onClick={this.openPage(href)}>
											{label}
										</StyledTopMenuItem>
									)
							)
						}
					</StyledTopLeftMenuContainer>
				)
		)
	};

	public render(): React.ReactElement {
		const {
			leftMenuItems,
			topMenuItems,
			children,
			logo,
			enableLeftMenu,
		} = this.props;
		const {
			open,
			maxLeftMenuItemCount,
		} = this.state;

		return (
			<React.Fragment>
				<StyledLeftMenu open={open} enableLeftMenu={enableLeftMenu}>
					{
						enableLeftMenu && (
							<StyledLeftMenuLogo>
								{logo}
							</StyledLeftMenuLogo>
						)
					}
					{
						leftMenuItems && leftMenuItems.map(
							({ label, href }, i) => (
								<StyledLeftMenuItem key={i} onClick={this.openPage(href)}>
									<StyledLeftMenuLabel open={open}>
										{label}
									</StyledLeftMenuLabel>
								</StyledLeftMenuItem>
							)
						)
					}
				</StyledLeftMenu>
				<StyledRightSide open={open} enableLeftMenu={enableLeftMenu}>
					<StyledTopMenu ref={this.topMenuRef} >
						{
							!enableLeftMenu && (
								<StyledTopMenuItemLogo>
									{logo}
								</StyledTopMenuItemLogo>
							)
						}
						{
							enableLeftMenu && (
								<StyledTopMenuItemHamburger onClick={this.toggleOpenLeftMenu}>
									<FontAwesomeIcon icon="bars" />
								</StyledTopMenuItemHamburger>
							)
						}
						{
							topMenuItems && topMenuItems.map(
								({ label, href, left = false }, i) =>
									!left && maxLeftMenuItemCount > i && (
										<StyledTopMenuItem key={i} onClick={this.openPage(href)}>
											{label}
										</StyledTopMenuItem>
									)
							)
						}
						{this.renderTopNavigation()}
					</StyledTopMenu>
					<StyledPageContent open={open} enableLeftMenu={enableLeftMenu} onClick={this.closeLeftMenu}>
						{children}
					</StyledPageContent>
				</StyledRightSide>

				<Modals />
				<Toasters />
			</React.Fragment>
		);
	}
}

export default PageLayout;
