import * as React from 'react';
import { is } from 'ramda';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'

import { IHTMLElement } from '@romanjaros/treejs-types/form';

import { StyledRadioButton, StyledRadioButtonItem, StyledRadioButtonTitle } from './style';
import { IOption } from './types';


export interface IProps<V = IOption> extends IHTMLElement<V> {
	customRender?: (option: IOption, selected: string) => React.ReactElement;
	horizontal?: boolean;
	options: IOption[];
	value?: string;
}

interface IState {
	value: string;
}

class RadioButton extends React.PureComponent<IProps, IState> {
	private static defaultProps = {
		horizontal: false,
	};

	public readonly state = {
		value: this.props.value,
	};

	public componentDidUpdate(prevProps: IProps, prevState: IState): void {
		let newValue = this.state.value;

		if (prevProps.value !== this.props.value) {
			newValue = this.props.value;
		} else if (prevState.value !== this.state.value) {
			newValue = this.state.value;
		} else {
			return;
		}

		this.setState({
			value: newValue,
		});
	}

	private handleClick = (option: IOption): any => (e: React.MouseEvent<HTMLInputElement>): void => {
		if (this.props.onClick) {
			this.props.onClick(option, e);
		}

		this.setState({
			value: option.code,
		})
	};

	private handleBlur = (option: IOption): any => (e: React.FocusEvent<HTMLInputElement>): void => {
		if (this.props.onBlur) {
			this.props.onBlur(option, e);
		}
	};

	public render(): React.ReactElement {
		const {
			title,
			options,
			horizontal,
			customRender,
		} = this.props;

		const optionElements = options.map(
			({ code, name }): React.ReactElement => (
				<StyledRadioButtonItem horizontal={horizontal} key={code}>
					<input
						id={`${this.props.name}_${code}`}
						name={name}
						type="radio"
						onClick={this.handleClick({ code, name })}
						onBlur={this.handleBlur({ code, name })}
					/>
					<label htmlFor={`${this.props.name}_${code}`}>
						{
							is(Function, customRender)
								? customRender({ code, name }, this.state.value)
								: (
									<React.Fragment>
										<FontAwesomeIcon
											size="lg"
											icon={
												this.state.value === code ? 'dot-circle' : 'circle'
											}
										/>
										<span>{name}</span>
									</React.Fragment>
								)
						}
					</label>
				</StyledRadioButtonItem>
			)
		);

		return (
			<StyledRadioButton>
				<StyledRadioButtonTitle>{title}</StyledRadioButtonTitle>
				{optionElements}
			</StyledRadioButton>
		);
	}
}

export default RadioButton;
