import * as React from 'react';
import { findIndex, propEq, propOr, filter, compose, prop, contains, toLower } from 'ramda';

import { isNilOrEmpty } from '@romanjaros/treejs-utils';
import { STATUS } from '@romanjaros/treejs-types/common';

import Section from '../../Section';
import Button from '../../Button';
import { IModalId } from '../../Modal/types';
import TextField from '../../TextField';

import { IOption } from '../types';
import { StyledSelectBoxOptions } from '../style';


interface IProps {
	autoComplete?: boolean;
	closeModal: (id: IModalId) => void;
	name: IModalId;
	onChange: (options: IOption) => void;
	onRemove: () => void;
	options: IOption[];
	selectedOption: IOption;
}

interface IState {
	index: number;
	searchResult?: IOption[];
}

class Modal extends React.Component<IProps, IState> {

	public readonly state = {
		index: 0,
		searchResult: this.props.options,
	};

	public componentDidMount(): void {
		window.addEventListener("keydown", this.handleKeyUp, { passive: false });

		if (!isNilOrEmpty(this.props.options)) {
			if (!isNilOrEmpty(this.props.selectedOption)) {
				const index = findIndex(propEq('code', this.props.selectedOption.code), this.props.options);
				if (index !== -1) {
					this.setState({
						index,
					})
				}
			}
		}
	}

	public componentWillUnmount(): void {
		window.removeEventListener("keydown", this.handleKeyUp);
	}

	private setCurrentOption = (optionIndex: number) => (): void => {
		this.setState({
			index: optionIndex,
		});
	};

	private setActiveOption = (option: IOption) => (): void => {
		this.props.onChange(option);
		this.props.closeModal(this.props.name);
	};

	private removeActiveOption = (): void => {
		this.props.onRemove();
		this.props.closeModal(this.props.name);
	};

	private handleKeyUp = (event: KeyboardEvent): void => {
		// event.preventDefault();
		let newIndex = this.state.index;

		switch (event.key) {
			case 'ArrowDown':
				newIndex += 1;
				break;
			case 'ArrowUp':
				newIndex -= 1;
				break;
			case 'Enter':
				this.setActiveOption(this.props.options[this.state.index])();
				return;
			default:
				return;
		}

		if (newIndex < 0) {
			newIndex = 0;
		} else if (newIndex > this.props.options.length - 1) {
			newIndex = this.props.options.length - 1;
		}

		this.setCurrentOption(newIndex)();
	};

	private handleSearch = (value: string) => {
		this.setState({
			searchResult: filter<IOption>(
				compose(
					contains(toLower(value)),
					toLower,
					prop('name'),
				)
			)(this.props.options),
		});
	}

	public render(): React.ReactElement {
		const {
			autoComplete,
		} = this.props;
		const {
			searchResult,
		} = this.state;

		return (
			<div>
				{
					autoComplete && (
						<Section>
							<TextField
								name="search"
								title="Hledat"
								onChange={this.handleSearch}
							/>
						</Section>
					)
				}
				<Section>
					{
						searchResult.map(
							({ code, name }, i) => (
								<StyledSelectBoxOptions
									key={code}
									isActive={this.state.index === i}
									isSelected={propOr('', 'code', this.props.selectedOption) === code}
									onClick={this.setActiveOption({ code, name })}
									onMouseEnter={this.setCurrentOption(i)}
								>
									{name}
								</StyledSelectBoxOptions>
							)
						)
					}
					{
						isNilOrEmpty(searchResult) && (
							<div>Nic k zobrazení</div>
						)
					}
				</Section>
				<Button
					title="Odebrat zvolenou hodnotu"
					status={STATUS.error}
					onClick={this.removeActiveOption}
				/>
			</div>
		);
	}
}

export default Modal;
